package com.example.fieldagent.ui.profile

import android.app.AlertDialog
import android.content.Intent
import android.content.res.Resources
import android.os.Bundle
import android.text.Editable
import android.text.InputType
import android.text.TextUtils
import android.text.TextWatcher
import android.util.Log
import android.view.Gravity
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.navigation.fragment.findNavController
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.R
import com.example.fieldagent.data.models.responses.Data
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.databinding.FragmentDamageListBinding
import com.example.fieldagent.databinding.FragmentProfileBinding
import com.example.fieldagent.ui.damagelist.DamageListFragmentVM
import com.example.fieldagent.ui.damagelist.adapter.DamageListAdapter
import com.example.fieldagent.ui.login.LoginActivity
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.USER_DATA
import com.example.fieldagent.utils.dialogs.ProgressDialog
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.isConnectedToInternet
import com.example.fieldagent.utils.showConfirmDialog
import com.example.fieldagent.utils.showSnackBar
import com.example.fieldagent.utils.showUpdateDialog
import com.example.fieldagent.utils.visible
import com.google.gson.Gson
import dagger.hilt.android.AndroidEntryPoint
import java.util.HashMap
import javax.inject.Inject
import androidx.core.text.layoutDirection


@AndroidEntryPoint
class ProfileFragment : Fragment() {


    private var _binding: FragmentProfileBinding? = null

    private val binding get() = _binding!!

    private  val viewModel: ProfileFragmentVM by viewModels()

    @Inject
    lateinit var prefsManager: PrefsManager
    private lateinit var progressDialog: ProgressDialog

    private var userId="";
    private var userName="";
    private var isPasswordVisible = false
    private var isConfirmPasswordVisible = false
    private var isClickName = false
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentProfileBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        initialize()
        listener()
        bindObservers()

    }

    private fun initialize(){
        progressDialog=ProgressDialog(requireActivity())
        val jsonString = prefsManager.getString(USER_DATA, "")
        val userData = Gson().fromJson(jsonString, Data::class.java)

        userId=userData.id.toString()
        binding.etName.setText(userData.name)
        userName=userData.name.toString()
        binding.etEmail.setText(userData.email)
        binding.etPhone.setText(userData.phoneNumber)

        val currentLocale = Resources.getSystem().configuration.locales[0]

        if (currentLocale.layoutDirection == View.LAYOUT_DIRECTION_RTL) {
            binding.etPassword.textDirection = View.TEXT_DIRECTION_RTL
            binding.etPassword.layoutDirection = View.LAYOUT_DIRECTION_RTL
            binding.etPassword.gravity = Gravity.END
            binding.etConfirmPassword.textDirection = View.TEXT_DIRECTION_RTL
            binding.etConfirmPassword.layoutDirection = View.LAYOUT_DIRECTION_RTL
            binding.etConfirmPassword.gravity = Gravity.END
        } else {
            binding.etPassword.textDirection = View.TEXT_DIRECTION_LTR
            binding.etPassword.layoutDirection = View.LAYOUT_DIRECTION_LTR
            binding.etPassword.gravity = Gravity.START
            binding.etConfirmPassword.textDirection = View.TEXT_DIRECTION_LTR
            binding.etConfirmPassword.layoutDirection = View.LAYOUT_DIRECTION_LTR
            binding.etConfirmPassword.gravity = Gravity.START
        }


    }



    private fun listener(){


        binding.ivLeft.setOnClickListener {
            findNavController().popBackStack()
        }
        binding.llLogoutBtn.setOnClickListener {


                prefsManager.removeAll()

                val intent = Intent(requireContext(), LoginActivity::class.java)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
                startActivity(intent)

        }

        binding.etName.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                val newName = s.toString()


                if (newName.isNotEmpty() && newName != userName) {
                    binding.btnName.isEnabled = true

                } else {
                    binding.btnName.isEnabled = false

                }
            }

            override fun afterTextChanged(s: Editable?) {}
        })




        binding.tilPassword.setEndIconOnClickListener {
            isPasswordVisible = !isPasswordVisible

            if (isPasswordVisible) {
                binding.etPassword.inputType = InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_VISIBLE_PASSWORD
                binding.tilPassword.setEndIconDrawable(R.drawable.eye_icon) // Your open eye icon
            } else {
                binding.etPassword.inputType = InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_PASSWORD
                binding.tilPassword.setEndIconDrawable(R.drawable.eye_close) // Your closed eye icon
            }

            binding.etPassword.setSelection(binding.etPassword.text?.length ?: 0)
        }

        binding.tilConfirmPassword.setEndIconOnClickListener {
            isConfirmPasswordVisible = !isConfirmPasswordVisible

            if (isConfirmPasswordVisible) {
                binding.etPassword.inputType = InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_VISIBLE_PASSWORD
                binding.tilConfirmPassword.setEndIconDrawable(R.drawable.eye_icon) // Your open eye icon
            } else {
                binding.etPassword.inputType = InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_PASSWORD
                binding.tilConfirmPassword.setEndIconDrawable(R.drawable.eye_close) // Your closed eye icon
            }

            binding.etConfirmPassword.setSelection(binding.etConfirmPassword.text?.length ?: 0)
        }


        val watcher = object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
                val password = binding.etPassword.text.toString()
                val confirmPassword = binding.etConfirmPassword.text.toString()

                val isValid = password.isNotEmpty() &&
                        confirmPassword.isNotEmpty() &&
                        password == confirmPassword

                binding.btnPassword.isEnabled = isValid
                binding.btnPassword.setBackgroundResource(
                    if (isValid) R.drawable.bg_button_enabled
                    else R.drawable.bg_button_disabled
                )
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
        }

        binding.etPassword.addTextChangedListener(watcher)
        binding.etConfirmPassword.addTextChangedListener(watcher)





        binding.btnName.setOnClickListener {

            if(!binding.btnName.isEnabled) return@setOnClickListener
            when{
                isConnectedToInternet(requireActivity(), true) -> {
                    isClickName=true;
                    val hashMap = HashMap<String, String>()
                    hashMap["name"] = binding.etName.text.toString()
                    viewModel.updateProfile(userId,hashMap)

                }
            }
        }
        binding.btnPassword.setOnClickListener {

            if(!binding.btnPassword.isEnabled) return@setOnClickListener
            when{
                isConnectedToInternet(requireActivity(), true) -> {
                    isClickName=false
                    val hashMap = HashMap<String, String>()
                    if (binding.etPassword.text.toString().isNotEmpty())
                    {
                        hashMap["password"] = binding.etPassword.text.toString()
                    }


                    viewModel.updateProfile(userId,hashMap)

                }
            }
        }

      /*  binding.btnAdd.setOnClickListener {

            when{
                binding.etName.text.isEmpty()->{
                    binding.etName.showSnackBar(getString(R.string.please_enter_agent_name))
                }
                *//*binding.etPassword.text.toString().isEmpty()->{
                    binding.etPassword.showSnackBar(getString(R.string.please_enter_password))
                }*//*

                binding.etPassword.text.toString().isNotEmpty() && !binding.etPassword.text.toString().matches(Regex("^(?=.*[A-Z])(?=.*\\d)(?=.*[@#\$%^&+=!]).{6,}$"))->{
                    binding.etPassword.showSnackBar(getString(R.string.please_enter_valid_password))
                }

                binding.etPassword.text.toString().isNotEmpty() && binding.etConfirmPassword.text.toString().isEmpty()->{
                    binding.etConfirmPassword.showSnackBar(getString(R.string.please_enter_confirm_password))
                }
                binding.etPassword.text.toString() != binding.etConfirmPassword.text.toString()->{
                    binding.etConfirmPassword.showSnackBar(getString(R.string.password_and_confirm_password_not_match))
                }
                isConnectedToInternet(requireActivity(), true) -> {
                    val hashMap = HashMap<String, String>()
                    hashMap["name"] = binding.etName.text.toString()
                    if (binding.etPassword.text.toString().isNotEmpty())
                    {
                        hashMap["password"] = binding.etPassword.text.toString()
                    }


                    viewModel.updateProfile(userId,hashMap)
                }


            }
        }*/
    }


    private fun bindObservers(){
        viewModel.profile.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {
                    progressDialog.setLoading(false)

                    if(isClickName)
                    {
                        val jsonString = prefsManager.getString(USER_DATA, "")
                        val userDataCheck = Gson().fromJson(jsonString, Data::class.java)


                        userDataCheck.name = binding.etName.text.toString()

                        val updatedUserDataJson = Gson().toJson(userDataCheck)
                        prefsManager.saveString(USER_DATA, updatedUserDataJson)
                        userName=binding.etName.text.toString()

                        binding.btnName.isEnabled = false
                        requireContext().showUpdateDialog(
                        title = getString(R.string.profile_updated_successfully)
                    ) {

                    }

                    }else{
                        requireContext().showUpdateDialog(
                            title = getString(R.string.password_updated_successfully)
                        ) {
                            prefsManager.removeAll()

                            val intent = Intent(requireContext(), LoginActivity::class.java)
                            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
                            startActivity(intent)
                        }
                    }



                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })

    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }


}